#include <string.h>
#include <math.h>
#include "stack-c.h"
//#include "c_INTERANN.h"

/*  Interface for INTERANN */

extern void c_INTERANN(long int nvaldat,double * Qdata, double * DATEDEPDATA, 
          double * NBVALCALC, double * DATEDEPCALC,double * SCOEFF, 
          double * PERCTL, double * Qcalc_Med, double * Qcalc_P1, 
          double * Qcalc_P2, double * Qcalc_Moy);

int c_intINTERANN(fname) 
     char *fname;
{
  static int l1, m1, n1, l2, m2, n2,  l3, m3, n3, l4,m4,n4,l5,m5,n5,m6,n6,l6,l7,
              l8,l9,l10;
  static int minlhs=1, maxlhs=4, minrhs=6, maxrhs=6;
  int nval;	
 

  /* Check number of inputs (rhs=3) and outputs (lhs=1) */
  CheckRhs(minrhs,maxrhs) ;
  CheckLhs(minlhs,maxlhs) ;

  /* Get inputs */
  GetRhsVar(1, "d", &m1, &n1, &l1);   /* Discharge values */
  GetRhsVar(2, "d", &m2, &n2, &l2);   /* Date of initial value of input vector */
  GetRhsVar(3, "d", &m3, &n3, &l3);   /* Nb of values*/
  GetRhsVar(4, "d", &m4, &n4, &l4);   /* Date of initial value of generated vector*/
  GetRhsVar(5, "d", &m5, &n5, &l5);   /* Nb of Fourrier Coeff*/
  GetRhsVar(6, "d", &m6, &n6, &l6);   /* Percentile desired */

  /* Check dimensions    */
  if (n1!=1) {
	     sciprint("%s: Qdata should a single column vector \r\n", "c_INTERANN");
    	 Error(999);
    	 return 0;
  }
  if ((m2!=1)|(m3!=1)|(m4!=1)|(m5!=1)|(m6!=1)|
      (n2!=1)|(n3!=1)|(n4!=1)|(n5!=1)|(n6!=1)) { 
	     sciprint("%s: DATEDEPDATA, NBVALCALC, DATEDEPCALC, SCOEFF and PERCTL should be single numbers \r\n", "c_INTERANN");
    	 Error(999);
    	 return 0;
  }

  /* Get the size of the vector to return */
  nval = (int)(*stk(l3));
	
  /* Create the variable */
  CreateVar(7, "d", &nval, &n1, &l7);   /* Median - INTERANN time series */
  CreateVar(8, "d", &nval, &n1, &l8);   /* INTERANN time series percentile PERCTL */
  CreateVar(9, "d", &nval, &n1, &l9);   /* INTERANN time series percentile 1-PERCTL */
  CreateVar(10, "d",&nval, &n1, &l10);  /* Mean - INTERANN time series */

  /* Call c_INTERANN */
  c_INTERANN(m1,stk(l1),stk(l2), stk(l3), stk(l4), stk(l5),stk(l6),stk(l7),
              stk(l8),stk(l9),stk(l10));

  /*  Return result  */
  LhsVar(1) = 7;
  LhsVar(2) = 8;
  LhsVar(3) = 9;
  LhsVar(4) = 10;
  return 0;
}
