#include <string.h>
#include <math.h>
#include "stack-c.h"
//#include "c_KNN1SIM.h"

/*  Interface for c_KNN1SIM */

extern void c_KNN1SIM(int nvalDMAT, int ncolDMAT, int nrand, 
  double * NBK, double * WINS, double * DMAT, double * WEI,
  double * DMATSELEC,  double * NNdep, double * RND,double * KNNSIM_NN);

int c_intKNN1SIM(fname) 
     char *fname;
{
  static int l1, m1, n1,l2,m2,n2,l3,m3,n3,l4,m4,n4,l5,m5,n5,l6,m6,n6,l7,m7,n7,l8;
  static int minlhs=1, maxlhs=1, minrhs=7, maxrhs=7;

  /* Check number of inputs (rhs=7) and outputs (lhs=2) */
  CheckRhs(minrhs,maxrhs) ;
  CheckLhs(minlhs,maxlhs) ;

  /* Get Qdata (#1) and Param (#2) and create C (#3) as double ("d") matrices */
  GetRhsVar(1, "d", &m1, &n1, &l1);   /* NBK > number of neighbours */
  GetRhsVar(2, "d", &m2, &n2, &l2);   /* WINS -> temporal window */
  GetRhsVar(3, "d", &m3, &n3, &l3);   /* DMAT -> source of the resampling */
  GetRhsVar(4, "d", &m4, &n4, &l4);   /* WEI -> Weight of the variables (Eucl.Dist.) */
  GetRhsVar(5, "d", &m5, &n5, &l5);   /* DMATSELEC -> selection of days in DMAT (>0 => selected) */  
  GetRhsVar(6, "d", &m6, &n6, &l6);   /* NNdep -> initial value resampled */
  GetRhsVar(7, "d", &m7, &n7, &l7);   /* RND -> random numbers */
  
  /* Output variable */
  CreateVar(8, "d", &m7, &n1, &l8); /* Matrix of the resampled data */

  /* Check dimensions  */  
  if (!(m1==1)|!(m2==1)|!(m6==1)|!(n1==1)|!(n2==1)|!(n6==1)) { 
    sciprint("%s: NBK, WINS and NNdep should be single values \r\n", "c_KNN1SIM");
    Error(999);
    return 0;}    
  if (!(n5==1)) { 
    sciprint("%s: DMATSELEC should be a column vector \r\n", "c_KNN1SIM");
    Error(999);
    return 0;}    
  if (!(m4==n3)) { 
    sciprint("%s: WEI should have the same number of lines than the number of columns in DMAT \r\n", "c_KNN1SIM");
    Error(999);
    return 0;}
  if (!(m5==m3)) { 
    sciprint("%s: DMAT and DMATSELEC should have the same number of lines \r\n", "c_KNN1SIM");
    Error(999);
    return 0;}
  if (m3*n3>200000) { 
    sciprint("%s: Too many data in DMAT matrix (NxP<200000)\r\n", "c_KNN1SIM");
    Error(999);
    return 0;}
  if (m7>200000) { 
    sciprint("%s: Too many random numbers (Nr<200000)\r\n", "c_KNN1SIM");
    Error(999);
    return 0;}

  /* Call c_KNN1SIM */
  c_KNN1SIM(m3,n3,m7,stk(l1),stk(l2),stk(l3),stk(l4),stk(l5),stk(l6),stk(l7),
      stk(l8));
    
  /*  Return result  */
  LhsVar(1) = 8;
  
  return 0;
}

