#include <math.h>

// Year of initial date value
#define YEARDEB 1900
#define YEARFIN 10000000

// Missing value
#define LAC -999

// Precision of numerical date values
#define PRECIS 0.01

/***********************************************
* Conversion of date from AAAAMMJJhhmm to Excel numerical format
* and vice-versa 
*
*	c_CONVDATE inputs :
*	nval 	= Number of value (nb of lines)
*	DATES 	= Vector Matrix (length=nval) containing date values
*	FORMAT	= Integer (0/1) indicating the conversion type :
*			0 : From calendar type (AAAAMMJJHHmm) to Excel numerical type (unit = day, 1 = 1/1/1900 00:00)
*			1 : From Excel numerical type to calendar type - WARNING valid only if YEARDEB=1900 !!!
*
*	output :
*	CONV 	= Converted date
*
************************************************/

/* Conversion function from date values expressed in year, month and day in numerical value */
double DATE2DOUBLE(int YEAR,int MONTH, int DAY)
{
	int i,nbj,nbja,nbjm,nbjmtab[12],checkDAY;

	// Number of day per month for normal year
	nbjmtab[0]=31;
	nbjmtab[1]=28;// Warning !!
	nbjmtab[2]=31;
	nbjmtab[3]=30;
	nbjmtab[4]=31;
	nbjmtab[5]=30;
	nbjmtab[6]=31;
	nbjmtab[7]=31;
	nbjmtab[8]=30;
	nbjmtab[9]=31;
	nbjmtab[10]=30;
	nbjmtab[11]=31;

	// No value returned for wrong values of YEAR, MONTH or DAY
	if(YEAR<YEARDEB)		{return LAC;}
	if((MONTH<1)|(MONTH>12))	{return LAC;}
	checkDAY=nbjmtab[MONTH-1];
	if((fmod(YEAR,4)==0)&((fmod(YEAR,100)!=0)|(fmod(YEAR,400)==0))&(MONTH==2)){checkDAY=29;};
	if((DAY<1)|(DAY>checkDAY))	{return LAC;}

	nbj=0;
	for(i=YEARDEB;i<YEAR;i++){
		nbja=365;
		//if(fmod(i,4)==0){nbja=366;} >> attention, l'anne 1900 n'est pas bisextile
		if((fmod(i,4)==0)&(i!=1900)){nbja=366;}
    nbj+=nbja;
	}
	for(i=1;i<MONTH;i++){
		nbjm=nbjmtab[i-1];
		if(((fmod(YEAR,4)==0)&(YEAR!=1900))&(i==2)){nbjm=29;};
		nbj+=nbjm;
	}
	
	return	(double)(nbj+DAY);

}

/* Rounding function :  */
int ROUND(double val)
{
	double valinf,valsup;
	valinf=floor(val);
	valsup=ceil(val);
	if(val<=valsup-PRECIS){return (int)(valinf);}
	else {return (int)(valsup);}

}

/* ---------- Main sub ----------------------- */
void c_CONVDATE(int nval, int *FORMAT, double *DATES, double *CONV)
{
	int i,format,min,hour,mday,mon,year,k;
	double temp,remday;

	// Initialisations
	for(i=0;i<nval;i++){CONV[i]=LAC;}
	format=FORMAT[0];

	// check of FORMAT value
	if((format!=0)&(format!=1)){exit(-1);}

	for(i=0;i<nval;i++){

		/* ~~~~~~ From calendar type to numerical ~~~~~~ */
		if(format==0){;
			year=(int)(DATES[i]/100000000);
			mon=(int)(DATES[i]/1000000-(double)year*100);
			mday=(int)(DATES[i]/10000-(double)year*10000-(double)mon*100);
			hour=(int)(DATES[i]/100-(double)year*1000000-mon*10000-(double)mday*100);
			min=(int)(DATES[i]-(double)year*100000000-(double)mon*1000000
					-(double)mday*10000-(double)hour*100);

			temp=DATE2DOUBLE(year,mon,mday);

			if(temp==LAC){CONV[i]=LAC;}
			else {CONV[i]=temp+(double)(hour)/24+(double)(min)/60/24;}		
		}

		/* ~~~~~~ From numerical to calendar ~~~~~~ */
		if(format==1){
			temp=DATES[i]-floor(DATES[i]);

			// Calculation of minute and hour value
			hour=ROUND(temp*24);
			min=(int)((temp*24-hour)*60);

			// Calculation of year value (valid only if YEARDEB=1900 !!!)
			remday=fmod(DATES[i],1461);
			year=(int)((DATES[i]-remday)/1461)*4+1900;
			if(remday>=366){year+=1+(int)((remday-366)/365);}
			//if((remday==0)|(remday==731)|(remday==1096)){year--;}
			//printf("remday = %5.0f year = %5d\n",remday,year);
			
			if (year<YEARFIN){
				// Calculation of month value
				mon=1;
				for (k=1;k<13;k++){if(DATE2DOUBLE(year,k,1)>DATES[i]){break;} mon=k;}
	
				// Calculation of day value
				mday = (int)(floor(DATES[i])-DATE2DOUBLE(year,mon,1)+1);
		    
				// final value	
				if(DATE2DOUBLE(year,mon,mday)!=LAC){
					CONV[i]=(double)(year)*100000000+(double)(mon)*1000000
						+(double)(mday)*10000+(double)(hour)*100+(double)(min);}
				}
			  else {CONV[i]=LAC;}

		}

		// Results display
		//printf("i=%3d DATES[i]=%17.6f CONV[i]=%17.3f remday=%17.6f year=%4d mon=%2d mday=%2d hour=%2d min=%2d\n",i,DATES[i],CONV[i],remday,year, mon, mday, hour, min);
	}
}
