#include <math.h>

/***********************************************
* Conversion of Temperature values in ETP time series (dated values)
*
*	c_ETPD inputs :
*	nval	= number of required values
*	LAT	= Latitude of the station location
*	TEMP	= Temperatures time series
*	DATE	= Date of the temperature time series
*
*	output :
*	ETP 	= PE time series
*
************************************************/

/* --- Time management functions and subs (cf. c_CONVDATE) -- */
void c_CONVDATE(int nval,int * FORMAT,double * DATES,double * CONV);

/* --- Rayonnement extraterrestre------------ 
*	JJ : jour julien
*	LAT: Latitude (radians)
*/
double RAYONEXT(double JJ, double LAT);

/* ---------- Main sub ----------------------- */

void c_ETPD(int nval,double LAT[], double TEMP[], double DATE[], double ETP[])
{
	int i,annCour,moCour,joCour,heCour,FORMAT[1];
	double JJ,lat,dateCour,INI[1],CONV[1],dateCourN,dateDebAnnN;

	// Initialisations
	if(LAT[0]>0){lat=LAT[0];}

	/*--- Parameter control ----------
	printf("\nnval=%5d lat=%5.2f\n",nval,lat);
	*/

	// Generation of the ETP time series
	for(i=0;i<nval;i++){

		// Get the current date in calendar format
		dateCour = DATE[i];
		annCour=(int)(dateCour/100000000);
		moCour=(int)((dateCour)/1000000-annCour*100);
		joCour=(int)((dateCour)/10000-annCour*10000-moCour*100);
		heCour=(int)((dateCour)/100-annCour*1000000-moCour*10000-joCour*100);

		// Calculates the julian day
		FORMAT[0]=0;

		INI[0]=annCour*pow(10,8)+pow(10,6)+pow(10,4);
		c_CONVDATE(1,FORMAT,INI,CONV);
		dateDebAnnN=CONV[0];		

		INI[0]=dateCour;
		c_CONVDATE(1,FORMAT,INI,CONV);
		dateCourN=CONV[0];		
		if((dateCourN==-999)|(dateDebAnnN==-999)){JJ=1;}
		else{JJ=floor(dateCourN-dateDebAnnN)+1;}


		// ETP calculation (daily values only)
		if(TEMP[i]>=-5){ETP[i]=RAYONEXT(JJ,lat)*(TEMP[i]+5)/100;}
		else{ETP[i]=0;}

		/* -- Control of ETP content -- 
		printf("i=%4d dateDebAnnN=%12.0f dateCourN=%12.0f JJ=%4.0f TEMP=%6.3f ETP=%6.3f\n",i,dateDebAnnN,dateCourN,JJ,TEMP[i],ETP[i]);
		*/

	}
	
}





