#include <math.h>
#include <sciprint.h>

// Number of season per year
#define NBSEAS 6

#define EPS 0.00000003



/***********************************************
* Daily rainfall generator (translated from Fortran code written by Claude Michel)
*	DATEDEP[0]	Starting date (format 199901010000)
*	RANDNB		2 column matrix containing random numbers [0..1]
*	NBVAL		2 numbers controlling the random generator (vector of 2 compon.)
*	PX		Parameters of the rainfall generator (matrix [NBSEAS+1]x8) 
*				PX(i,j) = generator parameter (i =[1..NBSEAS], j=[1..8]) 
*						calculated from rainfall time series. See c_GENEPLUIEPARAM
*				PX(NBSEAS+1,1)=average of positive rainfall
*	Pini[0]		Initial rainfall value (mm)
*
************************************************/

/* --- Time management sub -- */
void DEMAIN(int * j,int * m,int * a);

/***** CORE CODE ************/
void c_GENEPLUIE(int nbval,double DATEDEP[],double RANDNB[], double PX[],double Pini[], double GENEP[])
{
	int i,k,IP,IPP,SEAS;
	int ann;
	int moi;
	int jou;

	// Identification of the year, month and day in the month of the beginning of the simulation
	if(DATEDEP[0]>0){
		ann=(int)(DATEDEP[0]/100000000);
		moi=(int)(DATEDEP[0]/1000000-ann*100);
		jou= (int)(DATEDEP[0]/10000-ann*10000-moi*100);
	}

	// Initialisation of simulation
	GENEP[0] = Pini[0];

	// Test to check if there are missing values in PX
	for(k=0;k<(NBSEAS+1)*8-1;k++){
		if(PX[k]==-9.999){
			for(i=1;i<nbval;i++){
				GENEP[i]=-9.999;
			}
      sciprint("\tOne parameter has the value -9.99 -> Generation aborted\n");
			return;
		}		
	}

	// loop over each day of the simulation
	for(i=1;i<nbval;i++){
		// Identification of the current year, month and day in the month
		DEMAIN(&jou,&moi,&ann);
		
		// Season number
		SEAS = 1 + (int)((moi-1)*NBSEAS/12);
		
		// Determination of the generator state in the previous time step
		if(GENEP[i-1]==0){IP=1;}
		if((GENEP[i-1]>0) & (GENEP[i]<=PX[NBSEAS])){IP=2;}
		if(GENEP[i-1]>PX[NBSEAS]){IP=3;}

		// Calculation of rainfall for the time-step
		//   note that PX[i-1 + (NBSEAS+1)*(j-1)] == PX(i,j) with i and j starting at 1

		if(RANDNB[i]<=PX[SEAS-1+(NBSEAS+1)*(2*IP-1)]){GENEP[i]=0;}
		else{
			// IPP = 2 >> moderate rainfall - IPP = 3 >> important rainfall
			if(RANDNB[i]<=PX[SEAS-1+(NBSEAS+1)*2*IP]){IPP=2;}else{IPP=3;}

			if(IPP==3){GENEP[i]=PX[NBSEAS]-PX[SEAS-1]*log(RANDNB[i+nbval]);}
			else{
				if(fabs(PX[SEAS-1+(NBSEAS+1)*7])<EPS){GENEP[i]=PX[NBSEAS]*RANDNB[i+nbval];}
				else{GENEP[i]=PX[NBSEAS]/PX[SEAS-1+(NBSEAS+1)*7]*log(1+RANDNB[i+nbval]*(exp(PX[SEAS-1+(NBSEAS+1)*7])-1));}
			}		
		}

		//printf("[%5d]  %2d-%2d-%4d %10.6f\n",i,jou,moi,ann,GENEP[i]);
		
	}
}


/* DEMAIN routine :
* Permits to increase a set of 3 values (year,month and day) by one day
* used in c_GENEPLUIE
*   j  : rank of the day in the month
*   m  : month number
*   a  : year
*/
void DEMAIN(int * j,int * m,int * a)
{
	int JM[12],nbj,jj,mm,aa;
	JM[0]=31;JM[1]=28;JM[2]=31;JM[3]=30;JM[4]=31;JM[5]=30;
	JM[6]=31;JM[7]=31;JM[8]=30;JM[9]=31;JM[10]=30;JM[11]=31;
	//printf(" %2d-%2d-%4d ",*j,*m,*a);
	// Number of days in month
	nbj=JM[*m-1];
	if((*m==2)&(fmod(*a,4)==0)&(!(fmod(*a,100)==0)|(fmod(*a,400)==0))){nbj=29;}
	
	// Increment
	jj=*j+1;mm=*m;aa=*a;
      	
	// End of the month
	if(jj>nbj){
		jj=1;mm=*m+1;
		
		// End of the year
		if(mm>12){mm=1;aa=*a+1;}
	}
	
	// Result
	*j=jj;*m=mm;*a=aa;
	//printf(">> %2d-%2d-%4d\n",*j,*m,*a);
}
