#include <math.h>
#include <stdlib.h>

#define PI 3.14159265

// Number of fourrier coefficients for smoothing (365/2)
#define NB_COEFF_TOT 182

// Maximum number of years
#define NBANN_MAX 200

// Maximum size of vectors or matrices
#define SIZEMAXMAT 500   

// Maximum value in the sorting routine SORT
#define MAXVALSORT 100000 

/***********************************************
 PROBLEME AVEC LES VALEURS LACUNES !!!!!!!!!

* Generation of an interannual time series
*	Qdata		= Input data
*	DATEDEP		= Date of initial value for the generated vector
*	Qcalc_Med	= Generated vector (Median interannual daily discharge)
*	Qcalc_P1	= Generated vector (Qtile 10 interannual daily discharge)
*	Qcalc_P2	= Generated vector (Qtile 90 interannual daily discharge)
*
* CAUTION ! >> discharges are rounded at 0.001
*
************************************************/

/* --- Time management functions and subs (cf. c_GENEPLUIE) -- */
extern void DEMAIN(int * j,int * m,int * a);

/* EXPKENR routine :
* Calculate a half exponential Kernel based on the parameter sig
* used in c_INTERANN
*   a : Exponential constant
*   MaxSize : Maximum number of ordinates
*   EKERN : half Gaussian Kernel (Symetric Kernel)
*/
void EXPKERN(int nbORDMAX,double a, double * EKERN)
{
  int i;
  double Skern;
  
  //printf("OrdMax=%d a=%5.2f\n",nbORDMAX,a);
    
  // Check if the a factor is strictly positive
  if(a<0.0001){a=0.0001;}
  
  // Exponential Kernel ordinates
  Skern=0;
  for(i=0;i<nbORDMAX;i++){
    EKERN[i]=exp(-1*pow((i+0.5)/a,2));
    Skern+=EKERN[i];
  }

  // Normalisation to take into account the infinite tail
  for(i=0;i<nbORDMAX;i++){EKERN[i]/=(Skern*2);}
}

/* SORT routine :
* Permits to sort the values of INI vector in increasing order
* used in c_INTERANN
*   nval  : Number of value to sort
*   IN    : Values to sort
*   indexORD  : Rank of the sorted values in IN
*/
void SORTORD(int nval, double * IN, int * indexORD)
{
	int i,k,rk_minTemp, RKTEMP[SIZEMAXMAT];
	double MIN, TEMP[SIZEMAXMAT];
	//printf("nval=%4d\n",nval);
	// Control of vector size
  if(nval>SIZEMAXMAT){nval=SIZEMAXMAT;}
	
  // Initialisation
	for(i=0;i<nval;i++){TEMP[i]=IN[i];RKTEMP[i]=1;}

  for(i=0;i<nval;i++){
		MIN=MAXVALSORT;
		// Find the MIN in the remaining data
    for(k=0;k<nval-i;k++){if(TEMP[k]<MIN){MIN=TEMP[k];rk_minTemp=k;}}
    //printf("[%3d] %7.2f %7.2f %5d ",i,IN[i],MIN,rk_minTemp);
    
    // Store the rank (avoid the problem of identical values)
    for(k=0;k<nval;k++){if((IN[k]==MIN)&(RKTEMP[k]!=0)){*(indexORD+i)=k;RKTEMP[k]=0;break;}}
    //printf("  - %5d\n",*(indexORD+i));        

    // Eliminate the value from the dataset
		for(k=0;k<nval-i;k++){if(k>rk_minTemp){TEMP[k-1]=TEMP[k];}}		
		//printf(" %5d\n",*(indexORD+i));
	}   
}

/*---- CODE CODE --------------------------------------*/

void c_INTERANN(long int nvaldat,double * Qdata, double * DATEDEPDATA, 
          double * NBVALCALC, double * DATEDEPCALC,double * SCOEFF, 
          double * PERCTL, double * Qcalc_Med, double * Qcalc_P1, 
          double * Qcalc_P2, double * Qcalc_Moy)
{ 
	int i,k,l,kValPos,kkern1,kkern2;
	int a_debdata,m_debdata,j_debdata,a_debcalc,m_debcalc,j_debcalc,a,a_prec,m,j,pos,pos_ini;
	int JM[12],rk_Med,rk_P1,rk_P2,rkVAL[NBANN_MAX];
	long int nvalcalc;
	double Qinta_TAB[365][NBANN_MAX];
  // _1 >> raw values    _2 >> smoothed values
  double Qinta_Med_1[365],Qinta_P1_1[365],Qinta_P2_1[365];
	double Qinta_Med_2[365],Qinta_P1_2[365],Qinta_P2_2[365];
	double Qinta_Moy_1[365],Qinta_Moy_2[365];
	double VAL[NBANN_MAX],scoeff,ExpKernel[365],meanQ,perctl;
	
	// *** initialisations ***

  // Smoothing coefficient
  scoeff=SCOEFF[0];
  if(scoeff>183){scoeff=183;}
  if(scoeff<=1){scoeff=1;}
  
  // Smoothing coefficient
  perctl=PERCTL[0]/100;
  if(perctl>1){perctl=0.99;}
  if(perctl<=0){perctl=0.01;}

  // Exponential Kernel
  EXPKERN(365, scoeff, ExpKernel);	
  if(scoeff==1){for(i=0;i<365;i++){ExpKernel[i]=0;}ExpKernel[0]=0.5;}	
  //for(i=0;i<365;i++){printf("[%3d] %8.5f\n",i,ExpKernel[i]);}

  // Days in months
  JM[0]=31;JM[1]=28;JM[2]=31;JM[3]=30;JM[4]=31;JM[5]=30;
	JM[6]=31;JM[7]=31;JM[8]=30;JM[9]=31;JM[10]=30;JM[11]=31;

  // data and calculated value start date 
  if(DATEDEPDATA[0]<190001010000.0){DATEDEPDATA[0]=190001010000.0;}
	if(DATEDEPCALC[0]<190001010000.0){DATEDEPCALC[0]=190001010000.0;}

	a_debdata = (int)(DATEDEPDATA[0]/pow(10,8));
	m_debdata = (int)(DATEDEPDATA[0]/pow(10,6))-a_debdata*100;
	j_debdata = (int)(DATEDEPDATA[0]/pow(10,4))-m_debdata*100-a_debdata*10000;
	a = a_debdata;m = m_debdata;j = j_debdata;
	if((m==2)&(j==29)){m=2;j=28;}

	pos_ini = j-1; // position in the interannual vector (365 values)
	for(i=0;i<m-1;i++){pos_ini+=JM[i];}	pos=pos_ini;

	nvalcalc = (int)(NBVALCALC[0]);
	a_debcalc = (int)(DATEDEPCALC[0]/pow(10,8));
	m_debcalc = (int)(DATEDEPCALC[0]/pow(10,6))-a_debcalc*100;
	j_debcalc = (int)(DATEDEPCALC[0]/pow(10,4))-m_debcalc*100-a_debcalc*10000;
	
	/* ----------------------------------------------------------------------------------
	*
	*  Generation of the interannual vectors with smooting (365 values)
	*	
	* ----------------------------------------------------------------------------------- */	
	
	for(i=0;i<nvaldat;i++){
		// Generation of the table 
		Qinta_TAB[pos][a-a_debdata]=Qdata[i];
		//if(pos==0){printf("\n[%2d]",a-a_debdata);} printf(" %5.2f",Qdata[i]);

		// Exclusion of the 29th february and calculation of the position in the inter-annual vector
		a_prec=a;
		DEMAIN(&j,&m,&a);
		if(!((m==2)&(j==29))){pos++;}
		if(pos>364){pos=0;}
	}
	if(a>a_prec){a--;}
	if(a-a_debdata>NBANN_MAX){a=a_debdata+NBANN_MAX;}	

	// Calcuation of quantiles based on a sorting with SORT routine
	for(i=0;i<365;i++){
		// Vector to be sorted (double multiplied by 1000)
		kValPos=0;meanQ=0;
		for(k=0;k<a-a_debdata;k++){
			if(Qinta_TAB[i][k]>=0){VAL[kValPos]=Qinta_TAB[i][k];meanQ+=VAL[kValPos];kValPos++;}
		}
    		
		if(kValPos>0){
      // calculation of the mean
  		Qinta_Moy_1[i]=meanQ/kValPos;
  		
      // Sorting to calculate quantiles
  		//printf("\n[%4d]",i);
  		SORTORD(kValPos,VAL,rkVAL);
  		//for(k=0;k<a-a_debdata;k++){printf("%5.2f ",VAL[k]);}
  
  		// Get the median, qtile P1 and P2
  		rk_Med=(int)((double)(kValPos)*0.5)-1;
  		if(rk_Med>=kValPos){rk_Med=kValPos-1;}if(rk_Med<0){rk_Med=0;}
  		rk_P1=(int)((double)(kValPos)*perctl)-1;
  		if(rk_P1>=kValPos){rk_P1=kValPos-1;}if(rk_P1<0){rk_P1=0;}
      rk_P2=(int)((double)(kValPos)*(1-perctl))-1;
  		if(rk_P2>=kValPos){rk_P2=kValPos-1;}if(rk_P2<0){rk_P2=0;}
      		
  		Qinta_Med_1[i]=VAL[rkVAL[rk_Med]];
  		Qinta_P1_1[i]=VAL[rkVAL[rk_P1]];
  		Qinta_P2_1[i]=VAL[rkVAL[rk_P2]];
  		//printf("[%3d] %5.2f %5.2f %5.2f %5.2f\n",i,Qinta_Med_1[i],Qinta_P1_1[i],Qinta_P2_1[i],Qinta_Moy_1[i]);
		}
		else
		{
      Qinta_Moy_1[i]=-9.99;
  		Qinta_Med_1[i]=-9.99;
  		Qinta_P1_1[i]=-9.99;
  		Qinta_P2_1[i]=-9.99;
    }
	}

	// ----- Smoothing -----------
  for(i=0;i<365;i++){
      // Convolution for each value  
      //printf("\ni%3d : ",i);
      Qinta_Moy_2[i]=0;Qinta_Med_2[i]=0;Qinta_P1_2[i]=0;Qinta_P2_2[i]=0;
      for(k=0;k<183;k++){
          // Convolution index towards the right of the 365 elements vector
          if(i+k<365){kkern1=k;}else{kkern1=k-365;}
          // Convolution index towards the left of the 365 elements vector
          if(i-k>=0){kkern2=-k;}else{kkern2=365-k;}
          // Final product
          Qinta_Moy_2[i]+=Qinta_Moy_1[i+kkern1]*ExpKernel[k]+Qinta_Moy_1[i+kkern2]*ExpKernel[k];    
          Qinta_Med_2[i]+=Qinta_Med_1[i+kkern1]*ExpKernel[k]+Qinta_Med_1[i+kkern2]*ExpKernel[k];    
          Qinta_P1_2[i]+=Qinta_P1_1[i+kkern1]*ExpKernel[k]+Qinta_P1_1[i+kkern2]*ExpKernel[k];
          Qinta_P2_2[i]+=Qinta_P2_1[i+kkern1]*ExpKernel[k]+Qinta_P2_1[i+kkern2]*ExpKernel[k];
          //if(k==0){printf("[%4d] (%3d-%3d) %5.3f %5.3f\n",i,i+kkern1,i+kkern2,Qinta_Moy_1[i+kkern1],Qinta_Moy_1[i+kkern2]);}
                   
      }
      //printf("[%3d] %5.2f %5.2f %5.2f %5.2f\n",i,Qinta_Med_2[i],Qinta_P1_2[i],Qinta_P2_2[i],Qinta_Moy_2[i]);      
  }
  

	/* --------------------------------------------------
	*
	*  Generation of the calculated vector (NBVAL values)
	*	
	* -------------------------------------------------- */	
	a = a_debcalc;m = m_debcalc;j = j_debcalc;
	if((m==2)&(j==29)){m=2;j=28;}
	
	pos = j-1; // position in the interannual vector (365 values)
	for(i=0;i<m-1;i++){pos+=JM[i];}

	for(i=0;i<nvalcalc;i++){
		// Get the intere-annual value for the calendar day
		Qcalc_Moy[i] = Qinta_Moy_2[pos];	
		Qcalc_Med[i] = Qinta_Med_2[pos];	
		Qcalc_P1[i]= Qinta_P1_2[pos];	
		Qcalc_P2[i]= Qinta_P2_2[pos];	

		// Calculate the position in the inter-annual vector
		DEMAIN(&j,&m,&a);
		if(!((m==2)&(j==29))){pos++;}
		if(pos>364){pos=0;}		
	}

}
