#include <math.h>

// Size of X vector (WARNING : max UH ordinates = XSIZE-5 !!)
#define XSIZE 500
#define npX 0

/* !!!!!!!!!!!!!!!!!!
* BUG dans la formule du rservoir linaire !!
* dV/dt = E - S   et   S(t) = V(t)/C    C capacit du rserv  [temps] 
*
* Dpend de l'hypothse sur la structure temporelle de l'entre :
* Dans la suite w = exp(-Dt/C)
* 
* 1. concentre au dbut du pas de temps (V(0) = Vini(0) + I0)
*   En continu : S(t) = V(0)*w
*   En discret : S0 = V(0)*(1-w)
*                V(Dt) = V(0)*w 
*
* 2. Rparti sur tout le pas de temps (qq soit t, I(t)=I0)
*   En continu : S(t) = (V(0)-CI0)*w + CI0
*   En discret : S0 = V(0)*(1-w) + I0 * (Dt - C*(1-w))
*                V(Dt) = V(0)*w  + CI0 * (1-w)

/***********************************************
* Simplified propagation model (lag and route with 
*   simple lag -TYPE=1x- or GR4J UH2 -TYPE=2x- and a linear -TYPE=x1- or quadratic reservoir -TYPE=x2-)
*	Parameters :
*	X[0] = X1 time
*	X[1] = Rescaling factor for all models	
*	X[2] = Routing store reservoir size (>1) or Rescaling exponent (if TYPE = 13 or 23)
*	X[3] = Rescaling factor in the case of non-linear routing
* TYPE = 
*     10  X1 model alone
*     11  X1 model + linear reservoir
*     12  X1 model + quadratic reservoir
*     20  UH2 GR4J model alone
*     21  UH2 GR4J model + linear reservoir
*     22  UH2 GR4J model + quadratic reservoir
************************************************/

void UHDEC (int NPX,int NH,double * X,double C)
{
	int i;
	for(i=0;i<2*NH;i++){
		X[NPX+NH+i]=0;
		if(i==(int)(C))  {X[NPX+NH+i]=(floor)(C)+1-C;}
		if(i==(int)(C)+1){X[NPX+NH+i]=C-(floor)(C);}
	}				
}

void UH2 (int NPX, int NH,double * X,double C);


void c_PROPAG(long int nval,double TYPE[],double Xinp[],double Qdata[],double Qcalc[],double Rcalc[])
{
	int i,k,NH,typeUH,typeRES;
	double X1,X2,X3,X[XSIZE],V[XSIZE];
	double Qam,Qav,R,RTP;

	// Parameters
	typeUH=(int)(TYPE[0]/10);
	typeRES=(int)(TYPE[0])-typeUH*10;

	X1 = Xinp[0]; // Lag
	if(X1<0.01){X1=0.01;}

  // Rescaling factor for all models (transfer upstream dis. to downs.)
	X2 = Xinp[1];
  if(X2<0){X2=0;}if(X2>1000){X2=1000;}
  
  // UH shape parameter	
  X3 = Xinp[2];
	if((X3<1)&(typeRES==1)){X3=1;} // Linear reservoir
	if((X3<0)&(typeRES==2)){X3=0;} // Quadratic reservoir	

	// Initialisation
	NH=(int)(XSIZE/3)-npX-1;
	for(i=0;i<XSIZE;i++){X[i]=0;V[i]=0;}
	R=0;	
	// LAG model
	if(typeUH==1){UHDEC(npX,NH,X,X1);}
	if(typeUH==2){UH2(npX,NH,X,X1);}

	// Simulation
  // when negative input values occur, the last discharge is used 
	Qam=0;
	for(i=0;i<nval;i++){	
		if(Qdata[i]>=0){Qam=Qdata[i];;}
		
		// ......Upstream discharge UH (lag or other UH) ...........
		for (k=0;k<2*NH;k++){
			V[k]=V[1+k]+X[npX+NH+k]*Qam;
		}
		V[2*NH]=X[npX+3*NH-1]*Qam;

		// ...... Lag & Route reservoir .........		
		if(V[0]<0){V[0]=0;}
		RTP=R+V[0];if(RTP>=0){R=RTP;}

		// ...... Reservoir output for other routing options ...... 
		if(typeRES==0){Qav=V[0];}
		if(typeRES==1){Qav=R/X3;}
		if((typeRES==2)&(X3+R>0)){Qav=pow(R,2)/(R+X3);}
		if((typeRES==2)&(X3+R<=0)){Qav=0;}
		R-=Qav;
        
    if(!(typeRES==0)&!(typeRES==1)&!(typeRES==2)){Qav=-9.99;}		
    Qcalc[i]=X2*Qav;
		Rcalc[i]=R;
		
	}
}
